/*	proxylistin2.c -- 02/10/20 IF
**	Use secondary struct for each proxy inlet			  	
**	 
*/

#include "ext.h"			// Required for all Max external objects

void *this_class;			// Required. Global pointing to this class 
#define MAX 64
#define MAX_LIST 20

typedef struct _inlet
{
	void *p_proxy;
	int p_argc;
	t_atom p_argv[MAX_LIST];
} t_inlet;

typedef struct _prox	// Data structure for this object
{
	t_object p_ob;		// Must always be the first field; used by Max
	t_inlet p_in[MAX];
	short p_num;	// number of inlets
	long p_dummy;
	long p_inlet;	// inlet number
	void *p_out;		// Pointer to outlet
} t_prox;

// Prototypes for methods: you need a method for each message you want to respond to
void *prox_new(long num);
void prox_free(t_prox *x);
void prox_bang(t_prox *x);	// Method for message "int" in left inlet
void prox_int(t_prox *x, long n);
void prox_list(t_prox *x, t_symbol *s, short argc, t_atom *argv);

void main(void)
{
	// set up our class: create a class definition 
	setup((t_messlist **) &this_class, (method)prox_new, (method)prox_free, (short)sizeof(t_prox), 0L,
         A_DEFLONG, 0);	
	addbang((method)prox_bang);
	addint((method)prox_int);
	addmess((method)prox_list, "list", A_GIMME, 0);
}


void *prox_new(long num)
{
	t_prox *prox;
    int i;

post("Size: %d",  sizeof(t_prox));

	prox = (t_prox *)newobject(this_class);	// Create the new instance 
	if (num < 2)
		num = 2;
	prox->p_num = num; // total number of inlets (proxies)
    prox->p_out = listout(prox);	// Create outlet
    for (i = 0; i < MAX; i++)
    {
    	SETLONG(&prox->p_in[i].p_argv[0], 0);
    	SETLONG(&prox->p_in[i].p_argv[1], 0);
    	prox->p_in[i].p_argc = 2;
    }
    for (i = num - 1; i > 0; i--)
      prox->p_in[i].p_proxy = proxy_new(prox, (long)i, &prox->p_dummy);
	return(prox);				// Must return a pointer to the new instance 
}

void prox_bang(t_prox *x)
{
	x->p_inlet = proxy_getinlet((t_object *)x);
	outlet_list(x->p_out, 0L, x->p_in[x->p_inlet].p_argc, x->p_in[x->p_inlet].p_argv);
}

void prox_int(t_prox *x, long n)
{
	x->p_inlet = proxy_getinlet((t_object *)x);

	SETLONG(&x->p_in[x->p_inlet].p_argv[0], x->p_inlet);
	SETLONG(&x->p_in[x->p_inlet].p_argv[1], n);
	x->p_in[x->p_inlet].p_argc = 2;
	prox_bang(x);
}

void prox_list(t_prox *x, t_symbol *s, short argc, t_atom *argv)
{
	int i = (int)s;
	x->p_inlet = proxy_getinlet((t_object *)x);
	
	if (argc > MAX_LIST - 1)
 		argc = MAX_LIST - 1;
	SETLONG(&x->p_in[x->p_inlet].p_argv[0], x->p_inlet);
	x->p_in[x->p_inlet].p_argc = argc + 1;
//	x->p_argc[x->p_inlet] = argc + 1;
	for (i = 0; i < argc; i++)
	{
		x->p_in[x->p_inlet].p_argv[i + 1] = argv[i];
	}
	outlet_list(x->p_out, 0L, argc + 1, x->p_in[x->p_inlet].p_argv);
	prox_bang(x);
}  

void prox_free(t_prox *x)
{
	short i;
	
	for (i = 1; i < x->p_inlet; i++)
		freeobject(x->p_in[i].p_proxy);
}

